/*
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.server.endpoint.v1;

import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Response;

import io.swagger.v3.oas.annotations.Operation;

import org.springframework.stereotype.Component;

import static java.util.concurrent.TimeUnit.HOURS;

@CacheFor(value = 1, unit = HOURS)
@Path("/version")
@Component
public class VersionEndpoint {

    private final VersionService versionService;

    public VersionEndpoint(VersionService versionService) {
        this.versionService = versionService;
    }

    @GET
    @Produces("text/plain")
    @Operation(description = "Get the version")
    public Response doGetPlain() {
        return Response.ok(versionService.getVersion()).build();
    }

    @GET
    @Produces("application/json")
    @Operation(description = "Get the version as JSON specification")
    public Response doGetJson() {
        return Response.ok(versionService.getDetailed()).build();
    }
}
