/*
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.server.endpoint.v1.dto;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.google.errorprone.annotations.CanIgnoreReturnValue;
import com.google.errorprone.annotations.Var;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import javax.annotation.CheckReturnValue;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;
import org.immutables.value.Generated;

/**
 * Immutable implementation of {@link VConnectionConfiguration}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code ImmutableVConnectionConfiguration.builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableVConnectionConfiguration.of()}.
 */
@Generated(from = "VConnectionConfiguration", generator = "Immutables")
@SuppressWarnings({"all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@javax.annotation.processing.Generated("org.immutables.processor.ProxyProcessor")
@Immutable
@CheckReturnValue
final class ImmutableVConnectionConfiguration
    implements VConnectionConfiguration {
  private final String connectorId;
  private final String actionId;
  private final Map<String, String> options;

  private ImmutableVConnectionConfiguration(
      String connectorId,
      String actionId,
      Map<String, ? extends String> options) {
    this.connectorId = connectorId;
    this.actionId = actionId;
    this.options = createUnmodifiableMap(true, false, options);
  }

  private ImmutableVConnectionConfiguration(
      ImmutableVConnectionConfiguration original,
      String connectorId,
      String actionId,
      Map<String, String> options) {
    this.connectorId = connectorId;
    this.actionId = actionId;
    this.options = options;
  }

  /**
   * @return The value of the {@code connectorId} attribute
   */
  @JsonProperty("connectorId")
  @Override
  public String getConnectorId() {
    return connectorId;
  }

  /**
   * @return The value of the {@code actionId} attribute
   */
  @JsonProperty("actionId")
  @Override
  public String getActionId() {
    return actionId;
  }

  /**
   * @return The value of the {@code options} attribute
   */
  @JsonProperty("options")
  @Override
  public Map<String, String> getOptions() {
    return options;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link VConnectionConfiguration#getConnectorId() connectorId} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for connectorId (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableVConnectionConfiguration withConnectorId(String value) {
    if (Objects.equals(this.connectorId, value)) return this;
    return validate(new ImmutableVConnectionConfiguration(this, value, this.actionId, this.options));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link VConnectionConfiguration#getActionId() actionId} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for actionId (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableVConnectionConfiguration withActionId(String value) {
    if (Objects.equals(this.actionId, value)) return this;
    return validate(new ImmutableVConnectionConfiguration(this, this.connectorId, value, this.options));
  }

  /**
   * Copy the current immutable object by replacing the {@link VConnectionConfiguration#getOptions() options} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the options map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableVConnectionConfiguration withOptions(Map<String, ? extends String> entries) {
    if (this.options == entries) return this;
    Map<String, String> newValue = createUnmodifiableMap(true, false, entries);
    return validate(new ImmutableVConnectionConfiguration(this, this.connectorId, this.actionId, newValue));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableVConnectionConfiguration} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableVConnectionConfiguration
        && equalTo((ImmutableVConnectionConfiguration) another);
  }

  private boolean equalTo(ImmutableVConnectionConfiguration another) {
    return Objects.equals(connectorId, another.connectorId)
        && Objects.equals(actionId, another.actionId)
        && options.equals(another.options);
  }

  /**
   * Computes a hash code from attributes: {@code connectorId}, {@code actionId}, {@code options}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    @Var int h = 5381;
    h += (h << 5) + Objects.hashCode(connectorId);
    h += (h << 5) + Objects.hashCode(actionId);
    h += (h << 5) + options.hashCode();
    return h;
  }

  /**
   * Prints the immutable value {@code VConnectionConfiguration} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return "VConnectionConfiguration{"
        + "connectorId=" + connectorId
        + ", actionId=" + actionId
        + ", options=" + options
        + "}";
  }

  /**
   * Construct a new immutable {@code VConnectionConfiguration} instance.
   * @param connectorId The value for the {@code connectorId} attribute
   * @param actionId The value for the {@code actionId} attribute
   * @param options The value for the {@code options} attribute
   * @return An immutable VConnectionConfiguration instance
   */
  public static VConnectionConfiguration of(String connectorId, String actionId, Map<String, ? extends String> options) {
    return validate(new ImmutableVConnectionConfiguration(connectorId, actionId, options));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableVConnectionConfiguration validate(ImmutableVConnectionConfiguration instance) {
    Set<ConstraintViolation<ImmutableVConnectionConfiguration>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link VConnectionConfiguration} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable VConnectionConfiguration instance
   */
  public static VConnectionConfiguration copyOf(VConnectionConfiguration instance) {
    if (instance instanceof ImmutableVConnectionConfiguration) {
      return (ImmutableVConnectionConfiguration) instance;
    }
    return ImmutableVConnectionConfiguration.builder()
        .createFrom(instance)
        .build();
  }

  /**
   * Creates a builder for {@link VConnectionConfiguration VConnectionConfiguration}.
   * <pre>
   * ImmutableVConnectionConfiguration.builder()
   *    .connectorId(String | null) // nullable {@link VConnectionConfiguration#getConnectorId() connectorId}
   *    .actionId(String | null) // nullable {@link VConnectionConfiguration#getActionId() actionId}
   *    .putOption|putAllOptions(String =&gt; String) // {@link VConnectionConfiguration#getOptions() options} mappings
   *    .build();
   * </pre>
   * @return A new VConnectionConfiguration builder
   */
  public static ImmutableVConnectionConfiguration.Builder builder() {
    return new ImmutableVConnectionConfiguration.Builder();
  }

  /**
   * Builds instances of type {@link VConnectionConfiguration VConnectionConfiguration}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @Generated(from = "VConnectionConfiguration", generator = "Immutables")
  @NotThreadSafe
  public static final class Builder {
    private @Nullable String connectorId;
    private @Nullable String actionId;
    private Map<String, String> options = new LinkedHashMap<String, String>();

    private Builder() {
    }

    /**
     * Fill a builder with attribute values from the provided {@code VConnectionConfiguration} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * Collection elements and entries will be added, not replaced.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final Builder createFrom(VConnectionConfiguration instance) {
      Objects.requireNonNull(instance, "instance");
      String connectorIdValue = instance.getConnectorId();
      if (connectorIdValue != null) {
        connectorId(connectorIdValue);
      }
      String actionIdValue = instance.getActionId();
      if (actionIdValue != null) {
        actionId(actionIdValue);
      }
      putAllOptions(instance.getOptions());
      return this;
    }

    /**
     * Initializes the value for the {@link VConnectionConfiguration#getConnectorId() connectorId} attribute.
     * @param connectorId The value for connectorId (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("connectorId")
    public final Builder connectorId(String connectorId) {
      this.connectorId = connectorId;
      return this;
    }

    /**
     * Initializes the value for the {@link VConnectionConfiguration#getActionId() actionId} attribute.
     * @param actionId The value for actionId (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("actionId")
    public final Builder actionId(String actionId) {
      this.actionId = actionId;
      return this;
    }

    /**
     * Put one entry to the {@link VConnectionConfiguration#getOptions() options} map.
     * @param key The key in the options map
     * @param value The associated value in the options map
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final Builder putOption(String key, String value) {
      this.options.put(
          Objects.requireNonNull(key, "options key"),
          Objects.requireNonNull(value, "options value"));
      return this;
    }

    /**
     * Put one entry to the {@link VConnectionConfiguration#getOptions() options} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final Builder putOption(Map.Entry<String, ? extends String> entry) {
      String k = entry.getKey();
      String v = entry.getValue();
      this.options.put(
          Objects.requireNonNull(k, "options key"),
          Objects.requireNonNull(v, "options value"));
      return this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link VConnectionConfiguration#getOptions() options} map. Nulls are not permitted
     * @param entries The entries that will be added to the options map
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("options")
    public final Builder options(Map<String, ? extends String> entries) {
      this.options.clear();
      return putAllOptions(entries);
    }

    /**
     * Put all mappings from the specified map as entries to {@link VConnectionConfiguration#getOptions() options} map. Nulls are not permitted
     * @param entries The entries that will be added to the options map
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final Builder putAllOptions(Map<String, ? extends String> entries) {
      for (Map.Entry<String, ? extends String> e : entries.entrySet()) {
        String k = e.getKey();
        String v = e.getValue();
        this.options.put(
            Objects.requireNonNull(k, "options key"),
            Objects.requireNonNull(v, "options value"));
      }
      return this;
    }

    /**
     * Builds a new {@link VConnectionConfiguration VConnectionConfiguration}.
     * @return An immutable instance of VConnectionConfiguration
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public VConnectionConfiguration build() {
      return ImmutableVConnectionConfiguration.validate(new ImmutableVConnectionConfiguration(null, connectorId, actionId, createUnmodifiableMap(false, false, options)));
    }
  }

  private static <K, V> Map<K, V> createUnmodifiableMap(boolean checkNulls, boolean skipNulls, Map<? extends K, ? extends V> map) {
    switch (map.size()) {
    case 0: return Collections.emptyMap();
    case 1: {
      Map.Entry<? extends K, ? extends V> e = map.entrySet().iterator().next();
      K k = e.getKey();
      V v = e.getValue();
      if (checkNulls) {
        Objects.requireNonNull(k, "key");
        Objects.requireNonNull(v, "value");
      }
      if (skipNulls && (k == null || v == null)) {
        return Collections.emptyMap();
      }
      return Collections.singletonMap(k, v);
    }
    default: {
      Map<K, V> linkedMap = new LinkedHashMap<>(map.size());
      if (skipNulls || checkNulls) {
        for (Map.Entry<? extends K, ? extends V> e : map.entrySet()) {
          K k = e.getKey();
          V v = e.getValue();
          if (skipNulls) {
            if (k == null || v == null) continue;
          } else if (checkNulls) {
            Objects.requireNonNull(k, "key");
            Objects.requireNonNull(v, "value");
          }
          linkedMap.put(k, v);
        }
      } else {
        linkedMap.putAll(map);
      }
      return Collections.unmodifiableMap(linkedMap);
    }
    }
  }
}
