/*
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.server.endpoint.v1.handler.setup;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import com.google.errorprone.annotations.CanIgnoreReturnValue;
import com.google.errorprone.annotations.Var;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import io.syndesis.common.model.Kind;
import io.syndesis.common.model.WithConfigurationProperties;
import io.syndesis.common.model.WithConfiguredProperties;
import io.syndesis.common.model.WithKind;
import io.syndesis.common.model.WithName;
import io.syndesis.common.model.WithResourceId;
import io.syndesis.common.model.connection.ConfigurationProperty;
import io.syndesis.common.util.json.StringTrimmingConverter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import javax.annotation.CheckReturnValue;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;
import javax.annotation.concurrent.Immutable;
import javax.annotation.concurrent.NotThreadSafe;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;
import org.immutables.value.Generated;

/**
 * Immutable implementation of {@link OAuthApp}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new OAuthApp.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableOAuthApp.of()}.
 */
@Generated(from = "OAuthApp", generator = "Immutables")
@SuppressWarnings({"all"})
@SuppressFBWarnings
@ParametersAreNonnullByDefault
@javax.annotation.processing.Generated("org.immutables.processor.ProxyProcessor")
@Immutable
@CheckReturnValue
final class ImmutableOAuthApp implements OAuthApp {
  private final @Nullable String id;
  private final Kind kind;
  private final String name;
  private final Map<String, ConfigurationProperty> properties;
  private final Map<String, String> configuredProperties;
  private final String icon;
  private transient final boolean isDerived;

  private ImmutableOAuthApp(
      Optional<String> id,
      Kind kind,
      String name,
      Map<String, ? extends ConfigurationProperty> properties,
      Map<String, ? extends String> configuredProperties,
      String icon) {
    this.id = id.orElse(null);
    this.kind = kind;
    this.name = name;
    this.properties = createUnmodifiableMap(true, false, properties);
    initShim.configuredProperties(createUnmodifiableMap(true, false, configuredProperties));
    this.icon = icon;
    this.configuredProperties = initShim.getConfiguredProperties();
    this.isDerived = initShim.isDerived();
    this.initShim = null;
  }

  private ImmutableOAuthApp(ImmutableOAuthApp.Builder builder) {
    this.id = builder.id;
    this.kind = builder.kind;
    this.name = builder.name;
    this.properties = createUnmodifiableMap(false, false, builder.properties);
    this.icon = builder.icon;
    if (builder.configuredPropertiesIsSet()) {
      initShim.configuredProperties(createUnmodifiableMap(false, false, builder.configuredProperties));
    }
    this.configuredProperties = initShim.getConfiguredProperties();
    this.isDerived = initShim.isDerived();
    this.initShim = null;
  }

  private ImmutableOAuthApp(
      ImmutableOAuthApp original,
      @Nullable String id,
      Kind kind,
      String name,
      Map<String, ConfigurationProperty> properties,
      Map<String, String> configuredProperties,
      String icon) {
    this.id = id;
    this.kind = kind;
    this.name = name;
    this.properties = properties;
    initShim.configuredProperties(configuredProperties);
    this.icon = icon;
    this.configuredProperties = initShim.getConfiguredProperties();
    this.isDerived = initShim.isDerived();
    this.initShim = null;
  }

  private static final byte STAGE_INITIALIZING = -1;
  private static final byte STAGE_UNINITIALIZED = 0;
  private static final byte STAGE_INITIALIZED = 1;
  @SuppressWarnings("Immutable")
  private transient volatile InitShim initShim = new InitShim();

  @Generated(from = "OAuthApp", generator = "Immutables")
  private final class InitShim {
    private byte configuredPropertiesBuildStage = STAGE_UNINITIALIZED;
    private Map<String, String> configuredProperties;

    Map<String, String> getConfiguredProperties() {
      if (configuredPropertiesBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (configuredPropertiesBuildStage == STAGE_UNINITIALIZED) {
        configuredPropertiesBuildStage = STAGE_INITIALIZING;
        this.configuredProperties = createUnmodifiableMap(true, false, getConfiguredPropertiesInitialize());
        configuredPropertiesBuildStage = STAGE_INITIALIZED;
      }
      return this.configuredProperties;
    }

    void configuredProperties(Map<String, String> configuredProperties) {
      this.configuredProperties = configuredProperties;
      configuredPropertiesBuildStage = STAGE_INITIALIZED;
    }

    private byte isDerivedBuildStage = STAGE_UNINITIALIZED;
    private boolean isDerived;

    boolean isDerived() {
      if (isDerivedBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (isDerivedBuildStage == STAGE_UNINITIALIZED) {
        isDerivedBuildStage = STAGE_INITIALIZING;
        this.isDerived = isDerivedInitialize();
        isDerivedBuildStage = STAGE_INITIALIZED;
      }
      return this.isDerived;
    }

    private String formatInitCycleMessage() {
      List<String> attributes = new ArrayList<>();
      if (configuredPropertiesBuildStage == STAGE_INITIALIZING) attributes.add("configuredProperties");
      if (isDerivedBuildStage == STAGE_INITIALIZING) attributes.add("isDerived");
      return "Cannot build OAuthApp, attribute initializers form cycle " + attributes;
    }
  }

  private Map<String, String> getConfiguredPropertiesInitialize() {
    return OAuthApp.super.getConfiguredProperties();
  }

  private boolean isDerivedInitialize() {
    return OAuthApp.super.isDerived();
  }

  /**
   * @return The value of the {@code id} attribute
   */
  @JsonProperty("id")
  @Override
  public Optional<String> getId() {
    return Optional.ofNullable(id);
  }

  /**
   * @return The value of the {@code kind} attribute
   */
  @JsonProperty("kind")
  @JsonDeserialize(contentConverter = StringTrimmingConverter.class)
  @JsonIgnore
  @Override
  public Kind getKind() {
    return kind;
  }

  /**
   * @return The value of the {@code name} attribute
   */
  @JsonProperty("name")
  @JsonDeserialize(converter = StringTrimmingConverter.class)
  @Override
  public String getName() {
    return name;
  }

  /**
   * @return The value of the {@code properties} attribute
   */
  @JsonProperty("properties")
  @Override
  public Map<String, ConfigurationProperty> getProperties() {
    return properties;
  }

  /**
   * @return The value of the {@code configuredProperties} attribute
   */
  @JsonProperty("configuredProperties")
  @Override
  public Map<String, String> getConfiguredProperties() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getConfiguredProperties()
        : this.configuredProperties;
  }

  /**
   * @return The value of the {@code icon} attribute
   */
  @JsonProperty("icon")
  @Override
  public String getIcon() {
    return icon;
  }

  /**
   * @return The computed-at-construction value of the {@code isDerived} attribute
   */
  @JsonProperty("isDerived")
  @Override
  public boolean isDerived() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.isDerived()
        : this.isDerived;
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link OAuthApp#getId() id} attribute.
   * @param value The value for id
   * @return A modified copy of {@code this} object
   */
  public final ImmutableOAuthApp withId(String value) {
    @Nullable String newValue = Objects.requireNonNull(value, "id");
    if (Objects.equals(this.id, newValue)) return this;
    return validate(new ImmutableOAuthApp(this, newValue, this.kind, this.name, this.properties, this.configuredProperties, this.icon));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link OAuthApp#getId() id} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for id
   * @return A modified copy of {@code this} object
   */
  public final ImmutableOAuthApp withId(Optional<String> optional) {
    @Nullable String value = optional.orElse(null);
    if (Objects.equals(this.id, value)) return this;
    return validate(new ImmutableOAuthApp(this, value, this.kind, this.name, this.properties, this.configuredProperties, this.icon));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link OAuthApp#getKind() kind} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for kind (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOAuthApp withKind(Kind value) {
    if (this.kind == value) return this;
    if (Objects.equals(this.kind, value)) return this;
    return validate(new ImmutableOAuthApp(this, this.id, value, this.name, this.properties, this.configuredProperties, this.icon));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link OAuthApp#getName() name} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for name (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOAuthApp withName(String value) {
    if (Objects.equals(this.name, value)) return this;
    return validate(new ImmutableOAuthApp(this, this.id, this.kind, value, this.properties, this.configuredProperties, this.icon));
  }

  /**
   * Copy the current immutable object by replacing the {@link OAuthApp#getProperties() properties} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the properties map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableOAuthApp withProperties(Map<String, ? extends ConfigurationProperty> entries) {
    if (this.properties == entries) return this;
    Map<String, ConfigurationProperty> newValue = createUnmodifiableMap(true, false, entries);
    return validate(new ImmutableOAuthApp(this, this.id, this.kind, this.name, newValue, this.configuredProperties, this.icon));
  }

  /**
   * Copy the current immutable object by replacing the {@link OAuthApp#getConfiguredProperties() configuredProperties} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the configuredProperties map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableOAuthApp withConfiguredProperties(Map<String, ? extends String> entries) {
    if (this.configuredProperties == entries) return this;
    Map<String, String> newValue = createUnmodifiableMap(true, false, entries);
    return validate(new ImmutableOAuthApp(this, this.id, this.kind, this.name, this.properties, newValue, this.icon));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link OAuthApp#getIcon() icon} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for icon (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOAuthApp withIcon(String value) {
    if (Objects.equals(this.icon, value)) return this;
    return validate(new ImmutableOAuthApp(this, this.id, this.kind, this.name, this.properties, this.configuredProperties, value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableOAuthApp} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(@Nullable Object another) {
    if (this == another) return true;
    return another instanceof ImmutableOAuthApp
        && equalTo((ImmutableOAuthApp) another);
  }

  private boolean equalTo(ImmutableOAuthApp another) {
    return Objects.equals(id, another.id)
        && Objects.equals(kind, another.kind)
        && Objects.equals(name, another.name)
        && properties.equals(another.properties)
        && configuredProperties.equals(another.configuredProperties)
        && Objects.equals(icon, another.icon)
        && isDerived == another.isDerived;
  }

  /**
   * Computes a hash code from attributes: {@code id}, {@code kind}, {@code name}, {@code properties}, {@code configuredProperties}, {@code icon}, {@code isDerived}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    @Var int h = 5381;
    h += (h << 5) + Objects.hashCode(id);
    h += (h << 5) + Objects.hashCode(kind);
    h += (h << 5) + Objects.hashCode(name);
    h += (h << 5) + properties.hashCode();
    h += (h << 5) + configuredProperties.hashCode();
    h += (h << 5) + Objects.hashCode(icon);
    h += (h << 5) + Boolean.hashCode(isDerived);
    return h;
  }

  /**
   * Prints the immutable value {@code OAuthApp} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    StringBuilder builder = new StringBuilder("OAuthApp{");
    if (id != null) {
      builder.append("id=").append(id);
    }
    if (kind != null) {
      if (builder.length() > 9) builder.append(", ");
      builder.append("kind=").append(kind);
    }
    if (name != null) {
      if (builder.length() > 9) builder.append(", ");
      builder.append("name=").append(name);
    }
    if (builder.length() > 9) builder.append(", ");
    builder.append("properties=").append(properties);
    builder.append(", ");
    builder.append("configuredProperties=").append(configuredProperties);
    if (icon != null) {
      builder.append(", ");
      builder.append("icon=").append(icon);
    }
    builder.append(", ");
    builder.append("isDerived=").append(isDerived);
    return builder.append("}").toString();
  }

  /**
   * Construct a new immutable {@code OAuthApp} instance.
   * @param id The value for the {@code id} attribute
   * @param kind The value for the {@code kind} attribute
   * @param name The value for the {@code name} attribute
   * @param properties The value for the {@code properties} attribute
   * @param configuredProperties The value for the {@code configuredProperties} attribute
   * @param icon The value for the {@code icon} attribute
   * @return An immutable OAuthApp instance
   */
  public static OAuthApp of(Optional<String> id, Kind kind, String name, Map<String, ? extends ConfigurationProperty> properties, Map<String, ? extends String> configuredProperties, String icon) {
    return validate(new ImmutableOAuthApp(id, kind, name, properties, configuredProperties, icon));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableOAuthApp validate(ImmutableOAuthApp instance) {
    Set<ConstraintViolation<ImmutableOAuthApp>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link OAuthApp} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable OAuthApp instance
   */
  public static OAuthApp copyOf(OAuthApp instance) {
    if (instance instanceof ImmutableOAuthApp) {
      return (ImmutableOAuthApp) instance;
    }
    return new OAuthApp.Builder()
        .createFrom(instance)
        .build();
  }

  /**
   * Builds instances of type {@link OAuthApp OAuthApp}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @Generated(from = "OAuthApp", generator = "Immutables")
  @NotThreadSafe
  public static class Builder {
    private static final long OPT_BIT_CONFIGURED_PROPERTIES = 0x1L;
    private long optBits;

    private @Nullable String id;
    private @Nullable Kind kind;
    private @Nullable String name;
    private Map<String, ConfigurationProperty> properties = new LinkedHashMap<String, ConfigurationProperty>();
    private Map<String, String> configuredProperties = new LinkedHashMap<String, String>();
    private @Nullable String icon;

    /**
     * Creates a builder for {@link OAuthApp OAuthApp} instances.
     * <pre>
     * new OAuthApp.Builder()
     *    .id(String) // optional {@link OAuthApp#getId() id}
     *    .kind(io.syndesis.common.model.Kind | null) // nullable {@link OAuthApp#getKind() kind}
     *    .name(String | null) // nullable {@link OAuthApp#getName() name}
     *    .putProperty|putAllProperties(String =&gt; io.syndesis.common.model.connection.ConfigurationProperty) // {@link OAuthApp#getProperties() properties} mappings
     *    .putConfiguredProperty|putAllConfiguredProperties(String =&gt; String) // {@link OAuthApp#getConfiguredProperties() configuredProperties} mappings
     *    .icon(String | null) // nullable {@link OAuthApp#getIcon() icon}
     *    .build();
     * </pre>
     */
    public Builder() {
      if (!(this instanceof OAuthApp.Builder)) {
        throw new UnsupportedOperationException("Use: new OAuthApp.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.common.model.WithName} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final OAuthApp.Builder createFrom(WithName instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (OAuthApp.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.common.model.WithConfiguredProperties} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final OAuthApp.Builder createFrom(WithConfiguredProperties instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (OAuthApp.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.common.model.WithKind} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final OAuthApp.Builder createFrom(WithKind instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (OAuthApp.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.server.endpoint.v1.handler.setup.OAuthApp} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final OAuthApp.Builder createFrom(OAuthApp instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (OAuthApp.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.common.model.WithResourceId} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final OAuthApp.Builder createFrom(WithResourceId instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (OAuthApp.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.common.model.WithConfigurationProperties} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final OAuthApp.Builder createFrom(WithConfigurationProperties instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (OAuthApp.Builder) this;
    }

    private void from(Object object) {
      if (object instanceof WithName) {
        WithName instance = (WithName) object;
        String nameValue = instance.getName();
        if (nameValue != null) {
          name(nameValue);
        }
      }
      if (object instanceof WithConfiguredProperties) {
        WithConfiguredProperties instance = (WithConfiguredProperties) object;
        putAllConfiguredProperties(instance.getConfiguredProperties());
      }
      if (object instanceof WithKind) {
        WithKind instance = (WithKind) object;
        Kind kindValue = instance.getKind();
        if (kindValue != null) {
          kind(kindValue);
        }
      }
      if (object instanceof OAuthApp) {
        OAuthApp instance = (OAuthApp) object;
        String iconValue = instance.getIcon();
        if (iconValue != null) {
          icon(iconValue);
        }
      }
      if (object instanceof WithResourceId) {
        WithResourceId instance = (WithResourceId) object;
        Optional<String> idOptional = instance.getId();
        if (idOptional.isPresent()) {
          id(idOptional);
        }
      }
      if (object instanceof WithConfigurationProperties) {
        WithConfigurationProperties instance = (WithConfigurationProperties) object;
        putAllProperties(instance.getProperties());
      }
    }

    /**
     * Initializes the optional value {@link OAuthApp#getId() id} to id.
     * @param id The value for id
     * @return {@code this} builder for chained invocation
     */
    @CanIgnoreReturnValue 
    public final OAuthApp.Builder id(String id) {
      this.id = Objects.requireNonNull(id, "id");
      return (OAuthApp.Builder) this;
    }

    /**
     * Initializes the optional value {@link OAuthApp#getId() id} to id.
     * @param id The value for id
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("id")
    public final OAuthApp.Builder id(Optional<String> id) {
      this.id = id.orElse(null);
      return (OAuthApp.Builder) this;
    }

    /**
     * Initializes the value for the {@link OAuthApp#getKind() kind} attribute.
     * @param kind The value for kind (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("kind")
    @JsonDeserialize(contentConverter = StringTrimmingConverter.class)
    @JsonIgnore
    public final OAuthApp.Builder kind(Kind kind) {
      this.kind = kind;
      return (OAuthApp.Builder) this;
    }

    /**
     * Initializes the value for the {@link OAuthApp#getName() name} attribute.
     * @param name The value for name (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("name")
    @JsonDeserialize(converter = StringTrimmingConverter.class)
    public final OAuthApp.Builder name(String name) {
      this.name = name;
      return (OAuthApp.Builder) this;
    }

    /**
     * Put one entry to the {@link OAuthApp#getProperties() properties} map.
     * @param key The key in the properties map
     * @param value The associated value in the properties map
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final OAuthApp.Builder putProperty(String key, ConfigurationProperty value) {
      this.properties.put(
          Objects.requireNonNull(key, "properties key"),
          Objects.requireNonNull(value, "properties value"));
      return (OAuthApp.Builder) this;
    }

    /**
     * Put one entry to the {@link OAuthApp#getProperties() properties} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final OAuthApp.Builder putProperty(Map.Entry<String, ? extends ConfigurationProperty> entry) {
      String k = entry.getKey();
      ConfigurationProperty v = entry.getValue();
      this.properties.put(
          Objects.requireNonNull(k, "properties key"),
          Objects.requireNonNull(v, "properties value"));
      return (OAuthApp.Builder) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link OAuthApp#getProperties() properties} map. Nulls are not permitted
     * @param entries The entries that will be added to the properties map
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("properties")
    public final OAuthApp.Builder properties(Map<String, ? extends ConfigurationProperty> entries) {
      this.properties.clear();
      return putAllProperties(entries);
    }

    /**
     * Put all mappings from the specified map as entries to {@link OAuthApp#getProperties() properties} map. Nulls are not permitted
     * @param entries The entries that will be added to the properties map
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final OAuthApp.Builder putAllProperties(Map<String, ? extends ConfigurationProperty> entries) {
      for (Map.Entry<String, ? extends ConfigurationProperty> e : entries.entrySet()) {
        String k = e.getKey();
        ConfigurationProperty v = e.getValue();
        this.properties.put(
            Objects.requireNonNull(k, "properties key"),
            Objects.requireNonNull(v, "properties value"));
      }
      return (OAuthApp.Builder) this;
    }

    /**
     * Put one entry to the {@link OAuthApp#getConfiguredProperties() configuredProperties} map.
     * @param key The key in the configuredProperties map
     * @param value The associated value in the configuredProperties map
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final OAuthApp.Builder putConfiguredProperty(String key, String value) {
      this.configuredProperties.put(
          Objects.requireNonNull(key, "configuredProperties key"),
          Objects.requireNonNull(value, "configuredProperties value"));
      optBits |= OPT_BIT_CONFIGURED_PROPERTIES;
      return (OAuthApp.Builder) this;
    }

    /**
     * Put one entry to the {@link OAuthApp#getConfiguredProperties() configuredProperties} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final OAuthApp.Builder putConfiguredProperty(Map.Entry<String, ? extends String> entry) {
      String k = entry.getKey();
      String v = entry.getValue();
      this.configuredProperties.put(
          Objects.requireNonNull(k, "configuredProperties key"),
          Objects.requireNonNull(v, "configuredProperties value"));
      optBits |= OPT_BIT_CONFIGURED_PROPERTIES;
      return (OAuthApp.Builder) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link OAuthApp#getConfiguredProperties() configuredProperties} map. Nulls are not permitted
     * @param entries The entries that will be added to the configuredProperties map
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("configuredProperties")
    public final OAuthApp.Builder configuredProperties(Map<String, ? extends String> entries) {
      this.configuredProperties.clear();
      optBits |= OPT_BIT_CONFIGURED_PROPERTIES;
      return putAllConfiguredProperties(entries);
    }

    /**
     * Put all mappings from the specified map as entries to {@link OAuthApp#getConfiguredProperties() configuredProperties} map. Nulls are not permitted
     * @param entries The entries that will be added to the configuredProperties map
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    public final OAuthApp.Builder putAllConfiguredProperties(Map<String, ? extends String> entries) {
      for (Map.Entry<String, ? extends String> e : entries.entrySet()) {
        String k = e.getKey();
        String v = e.getValue();
        this.configuredProperties.put(
            Objects.requireNonNull(k, "configuredProperties key"),
            Objects.requireNonNull(v, "configuredProperties value"));
      }
      optBits |= OPT_BIT_CONFIGURED_PROPERTIES;
      return (OAuthApp.Builder) this;
    }

    /**
     * Initializes the value for the {@link OAuthApp#getIcon() icon} attribute.
     * @param icon The value for icon (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @CanIgnoreReturnValue 
    @JsonProperty("icon")
    public final OAuthApp.Builder icon(String icon) {
      this.icon = icon;
      return (OAuthApp.Builder) this;
    }

    /**
     * Builds a new {@link OAuthApp OAuthApp}.
     * @return An immutable instance of OAuthApp
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public OAuthApp build() {
      return ImmutableOAuthApp.validate(new ImmutableOAuthApp(this));
    }

    private boolean configuredPropertiesIsSet() {
      return (optBits & OPT_BIT_CONFIGURED_PROPERTIES) != 0;
    }
  }

  private static <K, V> Map<K, V> createUnmodifiableMap(boolean checkNulls, boolean skipNulls, Map<? extends K, ? extends V> map) {
    switch (map.size()) {
    case 0: return Collections.emptyMap();
    case 1: {
      Map.Entry<? extends K, ? extends V> e = map.entrySet().iterator().next();
      K k = e.getKey();
      V v = e.getValue();
      if (checkNulls) {
        Objects.requireNonNull(k, "key");
        Objects.requireNonNull(v, "value");
      }
      if (skipNulls && (k == null || v == null)) {
        return Collections.emptyMap();
      }
      return Collections.singletonMap(k, v);
    }
    default: {
      Map<K, V> linkedMap = new LinkedHashMap<>(map.size());
      if (skipNulls || checkNulls) {
        for (Map.Entry<? extends K, ? extends V> e : map.entrySet()) {
          K k = e.getKey();
          V v = e.getValue();
          if (skipNulls) {
            if (k == null || v == null) continue;
          } else if (checkNulls) {
            Objects.requireNonNull(k, "key");
            Objects.requireNonNull(v, "value");
          }
          linkedMap.put(k, v);
        }
      } else {
        linkedMap.putAll(map);
      }
      return Collections.unmodifiableMap(linkedMap);
    }
    }
  }
}
